package me.km.data;

import java.util.HashMap;
import me.km.api.Module;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.text.TextFormatting;

public class DataManager extends Module
{
    private final EntityData fallback;
    private final HashMap<Integer, EntityData> entityData;
    
    public DataManager(String mname, String prefix, TextFormatting color) 
    {
        super(mname, prefix, color);
        entityData = new HashMap<>();
        fallback = new EntityData(null);
    }
    
    public EntityData get(Entity ent)
    {
        if(ent == null)
        {
            return fallback;
        }
        return entityData.getOrDefault(ent.getEntityId(), fallback);
    }
    
    public EntityData getOrCreate(Entity ent)
    {
        EntityData data = entityData.getOrDefault(ent.getEntityId(), null);
        if(data == null)
        {
            return add(ent);
        }
        return data;
    }
    
    public EntityData add(Entity ent)
    {
        EntityData data = new EntityData(ent);
        entityData.put(ent.getEntityId(), data);
        return data;
    }
    
    public void onEntityDeath(Entity ent)
    {
        EntityData data = entityData.get(ent.getEntityId());
        if(data != null && data.onEntityDeath())
        {
            entityData.remove(ent.getEntityId());
        }
    }
    
    public void onPlayerLogOut(EntityPlayer p)
    {
        entityData.remove(p.getEntityId());
    }
    
    public void tick()
    {
        entityData.values().forEach(data -> data.tick());
    }
}
