package me.km.inventory;

import me.km.networking.ModPacketHandler;
import net.minecraft.advancements.CriteriaTriggers;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.IContainerListener;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.Slot;
import net.minecraft.inventory.SlotCrafting;
import net.minecraft.item.ItemStack;
import net.minecraft.network.play.server.SPacketSetSlot;
import net.minecraft.network.play.server.SPacketWindowItems;
import net.minecraft.network.play.server.SPacketWindowProperty;
import net.minecraft.util.NonNullList;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public class ContainerCampFire extends Container
{
    private final IInventory fireInv;
    private int[] cookTime;

    public ContainerCampFire(InventoryPlayer pInv, IInventory fireInv)
    {
        this.fireInv = fireInv;
        this.cookTime = new int[3];
        super.addSlotToContainer(new Slot(fireInv, 0, 52, 17));
        super.addSlotToContainer(new Slot(fireInv, 1, 52, 35));
        super.addSlotToContainer(new Slot(fireInv, 2, 52, 53));
        super.addSlotToContainer(new Slot(fireInv, 3, 108, 17));
        super.addSlotToContainer(new Slot(fireInv, 4, 108, 35));
        super.addSlotToContainer(new Slot(fireInv, 5, 108, 53));

        for(int i = 0; i < 3; i++)
        {
            for(int j = 0; j < 9; j++)
            {
                super.addSlotToContainer(new Slot(pInv, j + i * 9 + 9, 8 + j * 18, 84 + i * 18));
            }
        }

        for(int k = 0; k < 9; k++)
        {
            super.addSlotToContainer(new Slot(pInv, k, 8 + k * 18, 142));
        }
    }

    @Override
    public void addListener(IContainerListener listener)
    {
        super.addListener(listener);
        listener.sendAllWindowProperties(this, this.fireInv);
    }

    @Override
    public void detectAndSendChanges()
    {
        super.detectAndSendChanges();
        for(int i = 0; i < this.listeners.size(); ++i)
        {
            IContainerListener icontainerlistener = this.listeners.get(i);
            if(this.cookTime[0] != this.fireInv.getField(0))
            {
                icontainerlistener.sendWindowProperty(this, 0, this.fireInv.getField(0));
            }
            if(this.cookTime[1] != this.fireInv.getField(1))
            {
                icontainerlistener.sendWindowProperty(this, 1, this.fireInv.getField(1));
            }
            if(this.cookTime[2] != this.fireInv.getField(2))
            {
                icontainerlistener.sendWindowProperty(this, 2, this.fireInv.getField(2));
            }
        }
        this.cookTime[0] = this.fireInv.getField(0);
        this.cookTime[1] = this.fireInv.getField(1);
        this.cookTime[2] = this.fireInv.getField(2);
    }

    @SideOnly(Side.CLIENT)
    @Override
    public void updateProgressBar(int id, int data)
    {
        this.fireInv.setField(id, data);
    }

    @Override
    public boolean canInteractWith(EntityPlayer p)
    {
        return this.fireInv.isUsableByPlayer(p);
    }

    @Override
    public ItemStack transferStackInSlot(EntityPlayer p, int index)
    {
        return ItemStack.EMPTY;
    }
    
    public void openForPlayer(EntityPlayerMP p)
    {
        if(p.openContainer != p.inventoryContainer)
        {
            p.closeScreen();
        }
        p.getNextWindowId();
        p.openContainer = this;
        p.openContainer.windowId = p.currentWindowId;
        ModPacketHandler.sendCampFireInventory(p, windowId, fireInv.getName());
        p.openContainer.addListener(new IContainerListener() 
        {
            @Override
            public void sendAllContents(Container containerToSend, NonNullList<ItemStack> itemsList) 
            {
                p.connection.sendPacket(new SPacketWindowItems(containerToSend.windowId, itemsList));
                p.connection.sendPacket(new SPacketSetSlot(-1, -1, p.inventory.getItemStack()));
            }

            @Override
            public void sendSlotContents(Container containerToSend, int slotInd, ItemStack stack) 
            {
                if(!(containerToSend.getSlot(slotInd) instanceof SlotCrafting))
                {
                    if(containerToSend == p.inventoryContainer)
                    {
                        CriteriaTriggers.INVENTORY_CHANGED.trigger(p, p.inventory);
                    }
                    if(!p.isChangingQuantityOnly)
                    {
                        p.connection.sendPacket(new SPacketSetSlot(containerToSend.windowId, slotInd, stack));
                    }
                }
            }

            @Override
            public void sendWindowProperty(Container containerIn, int varToUpdate, int newValue) 
            {
                p.connection.sendPacket(new SPacketWindowProperty(containerIn.windowId, varToUpdate, newValue));
            }

            @Override
            public void sendAllWindowProperties(Container containerIn, IInventory inventory) 
            {
                for(int i = 0; i < inventory.getFieldCount(); i++)
                {
                    p.connection.sendPacket(new SPacketWindowProperty(containerIn.windowId, i, inventory.getField(i)));
                }
            }
        });
    }
}
