package me.km.inventory;

import me.km.networking.ModPacketHandler;
import net.minecraft.advancements.CriteriaTriggers;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.IContainerListener;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.Slot;
import net.minecraft.inventory.SlotCrafting;
import net.minecraft.item.ItemStack;
import net.minecraft.network.play.server.SPacketSetSlot;
import net.minecraft.network.play.server.SPacketWindowItems;
import net.minecraft.util.NonNullList;

public abstract class CustomContainerBase extends Container
{
    protected final EntityPlayer viewer;
    protected final boolean isServerSide;
    protected final InventoryBase inv;
    private final int numRows;

    protected CustomContainerBase(InventoryBase inv, EntityPlayer viewer)
    {
        // basic stuff
        this.viewer = viewer;
        this.isServerSide = viewer instanceof EntityPlayerMP;
        this.inv = inv;
        this.numRows = inv.getRows();
        int i = (this.numRows - 4) * 18;
        
        // inventory slots
        int counter = 0;
        for(int y = 0; y < this.numRows; y++)
        {
            for(int x = 0; x < 9; x++)
            {
                if(inv.isSlotValid(x, y))
                {
                    super.addSlotToContainer(new Slot(inv, counter, 8 + x * 18, 18 + y * 18));
                    counter++;
                }
            }
        }

        InventoryPlayer pInv = this.viewer.inventory;
        // plaver inventory slots
        for(int y = 0; y < 3; y++)
        {
            for(int x = 0; x < 9; x++)
            {
                super.addSlotToContainer(new Slot(pInv, x + y * 9 + 9, 8 + x * 18, 103 + y * 18 + i));
            }
        }
        for(int x = 0; x < 9; x++)
        {
            super.addSlotToContainer(new Slot(pInv, x, 8 + x * 18, 161 + i));
        }
    }

    public InventoryBase getInventoryBase() 
    {
        return inv;
    }
    
    public void closeSafe()
    {
        viewer.closeScreen();
    }
    
    public void openForPlayer()
    {
        if(isServerSide)
        {
            EntityPlayerMP p = (EntityPlayerMP) viewer;
            if(p.openContainer != p.inventoryContainer)
            {
                p.closeScreen();
            }
            p.getNextWindowId();
            p.openContainer = this;
            p.openContainer.windowId = p.currentWindowId;
            ModPacketHandler.sendCustomInventory(p, windowId, inv);
            p.openContainer.addListener(new IContainerListener() 
            {
                @Override
                public void sendAllContents(Container containerToSend, NonNullList<ItemStack> itemsList) 
                {
                    p.connection.sendPacket(new SPacketWindowItems(containerToSend.windowId, itemsList));
                    p.connection.sendPacket(new SPacketSetSlot(-1, -1, p.inventory.getItemStack()));
                }

                @Override
                public void sendSlotContents(Container containerToSend, int slotInd, ItemStack stack) 
                {
                    if(!(containerToSend.getSlot(slotInd) instanceof SlotCrafting))
                    {
                        if(containerToSend == p.inventoryContainer)
                        {
                            CriteriaTriggers.INVENTORY_CHANGED.trigger(p, p.inventory);
                        }
                        if(!p.isChangingQuantityOnly)
                        {
                            p.connection.sendPacket(new SPacketSetSlot(containerToSend.windowId, slotInd, stack));
                        }
                    }
                }

                @Override
                public void sendWindowProperty(Container containerIn, int varToUpdate, int newValue) 
                {
                }

                @Override
                public void sendAllWindowProperties(Container containerIn, IInventory inventory) 
                {
                }
            });
        }
    }

    @Override
    public boolean canInteractWith(EntityPlayer p)
    {
        return this.inv.isUsableByPlayer(p);
    }

    // ignore special slots stuff
    @Override
    public ItemStack transferStackInSlot(EntityPlayer p, int index)
    {
        ItemStack stack = ItemStack.EMPTY;
        Slot slot = this.inventorySlots.get(index);
        int size = inv.getSizeInventory();
        
        if(slot != null && slot.getHasStack())
        {
            ItemStack slotStack = slot.getStack();
            stack = slotStack.copy();
            if(index < size)
            {
                if(!this.mergeItemStack(slotStack, size, this.inventorySlots.size(), true))
                {
                    return ItemStack.EMPTY;
                }
            }
            else if(!this.mergeItemStack(slotStack, 0, size, false))
            {
                return ItemStack.EMPTY;
            }
            if(slotStack.isEmpty())
            {
                slot.putStack(ItemStack.EMPTY);
            }
            else
            {
                slot.onSlotChanged();
            }
        }
        return stack;
    }
    
    @Override
    public boolean canMergeSlot(ItemStack stack, Slot slot)
    {
        return inv.getSlotStatus(slot.slotNumber) == 1;
    }
    
    @Override
    public boolean canDragIntoSlot(Slot slot)
    {
        return inv.getSlotStatus(slot.slotNumber) == 1;
    }
    
    @Override
    protected boolean mergeItemStack(ItemStack stack, int startIndex, int endIndex, boolean reverseDirection)
    {
        boolean flag = false;
        int i = startIndex;

        if (reverseDirection)
        {
            i = endIndex - 1;
        }

        if (stack.isStackable())
        {
            while (!stack.isEmpty())
            {
                if (reverseDirection)
                {
                    if (i < startIndex)
                    {
                        break;
                    }
                }
                else if (i >= endIndex)
                {
                    break;
                }
                
                if(inv.getSlotStatus(i) == 1)
                {
                    Slot slot = this.inventorySlots.get(i);
                    ItemStack itemstack = slot.getStack();

                    if (!itemstack.isEmpty() && itemstack.getItem() == stack.getItem() && (!stack.getHasSubtypes() || stack.getMetadata() == itemstack.getMetadata()) && ItemStack.areItemStackTagsEqual(stack, itemstack))
                    {
                        int j = itemstack.getCount() + stack.getCount();
                        int maxSize = Math.min(slot.getSlotStackLimit(), stack.getMaxStackSize());

                        if (j <= maxSize)
                        {
                            stack.setCount(0);
                            itemstack.setCount(j);
                            slot.onSlotChanged();
                            flag = true;
                        }
                        else if (itemstack.getCount() < maxSize)
                        {
                            stack.shrink(maxSize - itemstack.getCount());
                            itemstack.setCount(maxSize);
                            slot.onSlotChanged();
                            flag = true;
                        }
                    }
                }

                if (reverseDirection)
                {
                    --i;
                }
                else
                {
                    ++i;
                }
            }
        }

        if (!stack.isEmpty())
        {
            if (reverseDirection)
            {
                i = endIndex - 1;
            }
            else
            {
                i = startIndex;
            }

            while (true)
            {
                if (reverseDirection)
                {
                    if (i < startIndex)
                    {
                        break;
                    }
                }
                else if (i >= endIndex)
                {
                    break;
                }

                if(inv.getSlotStatus(i) == 1)
                {
                    Slot slot1 = this.inventorySlots.get(i);
                    ItemStack itemstack1 = slot1.getStack();

                    if (itemstack1.isEmpty() && slot1.isItemValid(stack))
                    {
                        if (stack.getCount() > slot1.getSlotStackLimit())
                        {
                            slot1.putStack(stack.splitStack(slot1.getSlotStackLimit()));
                        }
                        else
                        {
                            slot1.putStack(stack.splitStack(stack.getCount()));
                        }

                        slot1.onSlotChanged();
                        flag = true;
                        break;
                    }
                }

                if (reverseDirection)
                {
                    --i;
                }
                else
                {
                    ++i;
                }
            }
        }

        return flag;
    }
}
