package me.km.items;

import java.util.LinkedList;
import java.util.List;
import me.km.blocks.CauldronColorMixer;
import net.minecraft.client.util.ITooltipFlag;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Items;
import net.minecraft.init.MobEffects;
import net.minecraft.item.Item;
import net.minecraft.item.ItemFishFood;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.nbt.NBTTagString;
import net.minecraft.potion.PotionEffect;
import net.minecraft.util.text.TextFormatting;
import net.minecraft.util.text.translation.I18n;
import net.minecraft.world.World;
import net.minecraftforge.common.capabilities.ICapabilityProvider;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public class ItemColoredSoup extends ItemFood
{
    public ItemColoredSoup(String name, String local) 
    {
        super(name, local, 0, 0, false);
        super.setMaxStackSize(1);
    }

    @Override
    public ItemStack onItemUseFinish(ItemStack stack, World w, EntityLivingBase liv)
    {
        super.onItemUseFinish(stack, w, liv);
        return new ItemStack(Items.BOWL);
    }

    @Override
    public ICapabilityProvider initCapabilities(ItemStack stack, NBTTagCompound nbt) 
    {
        getTagCompound(stack);
        return null;
    }  
    
    @SideOnly(Side.CLIENT)
    @Override
    public void addInformation(ItemStack stack, World w, List<String> lore, ITooltipFlag flag) 
    {
        String text = getLocalizedIngredientList(stack);
        if(!text.isEmpty())
        {
            lore.add(1, text);
        }
    }
    
    private NBTTagCompound getTagCompound(ItemStack stack)
    {
        NBTTagCompound com = stack.getTagCompound();
        if(com == null)
        {
            com = new NBTTagCompound();
            com.setInteger("color", CauldronColorMixer.DEFAULT.getRGB());
            com.setTag("ingredients", new NBTTagList());
            com.setFloat("saturation", 0);
            com.setInteger("food", 0);
            stack.setTagCompound(com);
        }
        else
        {
            if(!com.hasKey("color"))
            {
                com.setInteger("color", CauldronColorMixer.DEFAULT.getRGB());   
            }
            if(!com.hasKey("ingredients", 9))
            {
                com.setTag("ingredients", new NBTTagList());   
            }
            if(!com.hasKey("saturation"))
            {
                com.setFloat("saturation", 0);   
            }
            if(!com.hasKey("food"))
            {
                com.setInteger("food", 0);   
            }
        }
        return com;
    }
    
    @SuppressWarnings(value = "")
    public void addIngredient(ItemStack stew, Iterable<ItemStack> stacks)     
    {
        NBTTagCompound com = stew.getTagCompound();
        NBTTagList list = com.getTagList("ingredients", 8);
        for(ItemStack stack : stacks)
        {
            list.appendTag(new NBTTagString(stack.getUnlocalizedName() + ".name"));
        }
    }
    
    @SideOnly(Side.CLIENT)
    @SuppressWarnings(value = "")
    public String getLocalizedIngredientList(ItemStack stack)
    {
        NBTTagCompound com = stack.getTagCompound();
        NBTTagList list = com.getTagList("ingredients", 8);
        switch(list.tagCount())
        {
            case 0: return "";
            case 1: return TextFormatting.GRAY + I18n.translateToLocal(list.getStringTagAt(0));
            default:
                StringBuilder sb = new StringBuilder(TextFormatting.GRAY.toString());
                sb.append(I18n.translateToLocal(list.getStringTagAt(0)));
                for(int i = 1; i < list.tagCount(); i++)
                {
                    sb.append(", ");
                    sb.append(I18n.translateToLocal(list.getStringTagAt(i)));
                }
                return sb.toString();
        }
    }

    public void setColor(ItemStack stack, int color)
    {
        getTagCompound(stack).setInteger("color", color);
    }
    
    public int getColor(ItemStack stack)
    {
        return getTagCompound(stack).getInteger("color");
    }
    
    public void setFoodStats(ItemStack stack, LinkedList<ItemStack> ingredients)
    {
        float saturation = 0;
        int food = 0;
        
        for(ItemStack item : ingredients)
        {
            switch(Item.getIdFromItem(item.getItem()))
            {
                case 86: // pumpkin
                    food += 6;
                    saturation += 0.3F;
                    break;  
                case 260: // apple
                    food += 4;
                    saturation += 0.3F;
                    break;   
                case 322: // golden Apple
                    food += 4;
                    saturation += 1.2F;
                    break;   
                case 435: // beetroot Seeds
                case 362: // melon Seeds
                case 361: // pumpkin Seeds
                case 295: // wheat Seeds
                    food += 1;
                    saturation += 0.1F;
                    break; 
                case 363: // beef
                case 319: // pork
                    food += 8;
                    saturation += 0.8F;
                    break;             
                case 349: // fish
                    switch(ItemFishFood.FishType.byItemStack(item))
                    {
                        case CLOWNFISH:
                            food += 1;
                            saturation += 0.1F;
                            break;
                        case COD:
                            food += 5;
                            saturation += 0.6F;
                            break;
                        case PUFFERFISH:
                            food -= 75;
                            saturation -= 0.8F;
                            break;
                        case SALMON:
                            food += 6;
                            saturation += 0.8F;
                            break;
                    }
                    break; 
                case 353: // sugar
                    food += 1;
                    saturation += 0.1F;
                    break;  
                case 382: // golden melon
                case 360: // melon
                    food += 2;
                    saturation += 0.3F;
                    break; 
                case 365: // chicken
                    food += 6;
                    saturation += 0.6F;
                    break; 
                case 394: // poison potato
                case 375: // spider eye
                    food += -25;
                    saturation -= 0.2F;
                    break; 
                case 414: // rabbit foot
                case 372: // nether wart
                case 376: // spider eye fermented
                    food += 1;
                    saturation += 0.1F;
                    break; 
                /*case 378: // magma cream    
                case 377: // blaze powder
                    food -= 1;
                    saturation -= 0.1F;
                    break; */
                case 396: // golden carrot
                case 391: // carrot
                    food += 3;
                    saturation += 0.6F;
                    break; 
                case 411: // rabbit
                case 392: // potato
                    food += 5;
                    saturation += 0.6F;
                    break;     
                case 423: // mutton
                    food += 6;
                    saturation += 0.8F;
                    break; 
                case 434: // beetroot
                    food += 1;
                    saturation += 0.6F;
                    break; 
            }
        }
        
        getTagCompound(stack).setFloat("saturation", saturation);
        getTagCompound(stack).setInteger("food", food);
    }

    @Override
    public float getSaturationModifier(ItemStack stack) 
    {
        return getTagCompound(stack).getFloat("saturation");
    }

    @Override
    public int getHealAmount(ItemStack stack) 
    {
        int food = getTagCompound(stack).getInteger("food");
        if(food < 0)
        {
            return 0;
        }
        return food;
    }
    
    @Override
    protected void onFoodEaten(ItemStack stack, World worldIn, EntityPlayer player)
    {
        if(!worldIn.isRemote)
        {
            int food = getTagCompound(stack).getInteger("food");
            if(food < 0)
            {
                player.addPotionEffect(new PotionEffect(MobEffects.POISON, 100, (-food) / 25));
            }
        }
    }
}
