package me.km.jobsystem;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.TreeMap;
import java.util.stream.Collectors;
import me.km.skills.Skill;
import net.minecraft.block.Block;
import net.minecraft.item.Item;

public class Job 
{
    private final HashSet<Block> preferedBlocks;
    private final HashMap<Item, Byte> craftingRecipes;
    private final String name;
    private final HashMap<Skill, TreeMap<Byte, Byte>> skillmap;
    
    public Job(String name)
    {
        this.name = name;
        
        preferedBlocks = new HashSet<>();
        skillmap = new HashMap<>();
        craftingRecipes = new HashMap<>();
    }
    
    // -------------------------------------------------------------------------
    // Allgemeines
    // -------------------------------------------------------------------------
    
    public String getName()
    {
        return name;
    }
    
    // -------------------------------------------------------------------------
    // Materialien
    // -------------------------------------------------------------------------
    
    public void registerPreferedBlock(Block b)
    {
        if(b == null)
        {
            throw new NullPointerException();
        }
        preferedBlocks.add(b);
    }
    
    public HashSet<Block> getPreferedBlocks()
    {
        return preferedBlocks;
    }
    
    public boolean hasPreferedMaterial(Block b)
    {
        return preferedBlocks.contains(b);
    }
    
    // -------------------------------------------------------------------------
    // Rezepte
    // -------------------------------------------------------------------------
    
    public void registerRecipe(Item item, byte b)
    {
        if(item == null)
        {
            throw new NullPointerException();
        }
        craftingRecipes.put(item, b);
    }
    
    public Boolean hasRecipe(Item item, int level)
    {
        Byte b = craftingRecipes.get(item);
        return !(b == null || b > level);
    }
    
    public Set<Item> getRecipes(int level)
    {
        return craftingRecipes.entrySet().stream()
                .filter(e -> e.getValue() <= level).map(e -> e.getKey())
                .collect(Collectors.toSet());
    }
    
    // -------------------------------------------------------------------------
    // Skills
    // -------------------------------------------------------------------------
    
    public void registerSkillOnLevel(Skill skill, byte skillLevel, byte level)
    {
        if(skill == null)
        {
            throw new NullPointerException();
        }
        TreeMap<Byte, Byte> map = skillmap.get(skill);
        if(map == null)
        {
            map = new TreeMap<>();
            skillmap.put(skill, map);
        }
        map.put(level, skillLevel);
    }
    
    public byte getSkillLevel(Skill skill, byte level)
    {
        try
        {
            Byte b = skillmap.get(skill).floorEntry(level).getValue();
            return b == null ? 0 : b;
        }
        catch(NullPointerException ex)
        {
            return 0;
        }
    }
    
    public HashMap<Skill, Byte> getSkillMap(byte level)
    {
        //System.out.println("SKILLMAP CONTAINS " + skillmap.size() + " ELEMENTS");
        HashMap<Skill, Byte> map = new HashMap<>();
        if(level < 1)
        {
            return map;
        }
        skillmap.entrySet().forEach(e -> 
        {
            Byte b = e.getValue().floorEntry(level).getValue();
            if(b == null)
            {
            }
            else if(b > 0)
            {
                map.put(e.getKey(), b);
            }
        });
        return map;
    } 
}
