package me.km.skills;

import me.km.effects.Effect;
import me.km.utils.ItemStackBuilder;
import me.km.utils.ItemStackUtils;
import me.km.utils.ItemStackUtils.ItemFlag;
import net.minecraft.item.ItemStack;

public class Skill 
{ 
    private final int id;
    
    private final ItemStack stack;
    private final Effect effect;
    private final String name;
    
    private final int manaConstant;
    private final int manaFactor;
    private final int cooldownConstant;
    private final int cooldownFactor;
    
    public Skill(int id, ItemStack stack, Effect eff, String name, String explanation, 
            int manaConstant, int manaFactor, int cooldownConstant, int cooldownFactor)
    {
        this.id = id;
        ItemStackUtils.addItemFlag(stack, ItemFlag.HIDE_ATTRIBUTES, ItemFlag.HIDE_ENCHANTS, ItemFlag.HIDE_POTION_EFFECTS, ItemFlag.HIDE_UNBREAKABLE);
        stack.setStackDisplayName("§6" + name);
        ItemStackUtils.setLore(stack, ItemStackBuilder.buildLimitedLore(explanation, 25, "§7"));
        this.stack = stack;
        this.effect = eff;
        this.name = name;
        this.manaConstant = manaConstant;
        this.manaFactor = manaFactor;
        this.cooldownConstant = cooldownConstant;
        this.cooldownFactor = cooldownFactor;
    }

    public ItemStack getItemStack(int power)
    {
        int mana = getMana(power);
        stack.setCount(power);
        if(mana == 0)
        {
            return stack.copy();
        }
        return new ItemStackBuilder(stack, power).addToName(" §3(" + mana +")").build();
    }
    
    public int getMana(int power)
    {
        return manaConstant + power * manaFactor;
    }
    
    public int getCooldown(int power)
    {
        return cooldownConstant + power * cooldownFactor;
    }
    
    public boolean isActive()
    {
        return effect.isActive();
    }
    
    public Effect getEffect()
    {
        return effect;
    }
    
    public String getName()
    {
        return name;
    }

    public int getId() 
    {
        return id;
    }

    @Override
    public int hashCode() 
    {
        return id;
    }

    @Override
    public boolean equals(Object o) 
    {
        if(this == o) 
        {
            return true;
        }
        if (o == null) 
        {
            return false;
        }
        if(Skill.class != o.getClass()) 
        {
            return false;
        }
        return this.id == ((Skill) o).id;
    }
}
